-- ##################################################################
--
-- CreateDeferredSegments.sql
--
-- Copyright (c) 2013 Oracle, All rights reserved.
--
--    NAME
--
--      CreateDeferredSegments.sql
--
--    DESCRIPTION
--
--      This script materializes deferred segments in a given database schema using
--      the procedure DBMS_SPACE_ADMIN.MATERIALIZE_DEFERRED_SEGMENTS
--
--      Supported Oracle Release(s): 11.2.0.2 and above
--
--    USAGE
--
--      Connect to the database as database administrator
--      
--      SQL> @CreateDeferredSegments <schema>
--      SQL> @CreateDeferredSegments <schema> <table>
--
--      Examples:
--      1) sqlplus / as sysdba
--         SQL> @CreateDeferredSegments sapsr3
--
--      2) sqlplus / as sysdba @CreateDeferredSegments sapsr3
--
--    CHANGE HISTORY
--
--    MM/DD/YYYY - Change
--    04/17/2013 - created
--
-- ##################################################################

-- SQL*PLUS SETTINGS ------------------------------------------------
SET LINESIZE 1100
SET PAGESIZE 1000

SET VERIFY       OFF
SET HEADING      OFF
SET FEEDBACK     OFF
SET SERVEROUTPUT OFF

WHENEVER SQLERROR EXIT SQL.SQLCODE

-- INPUT VARIABLES - INITIALIZATION ----------------------------------
-- This script can use up to 2 input parameters.
-- The following construction initializes all input variables.
-- This avoids that the user is prompted for input.
-- http://www.freelists.org/post/oracle-l/sqlplus-substitution-variables,8

COL 1 NEW_VALUE 1
COL 2 NEW_VALUE 2

-- Condition 'where rownum = 0' is always FALSE
-- -> only variables which are not yet initialized get initialized
SELECT NULL "1", NULL "2" FROM DUAL WHERE ROWNUM = 0
/
  
-- Display / Test input variables -----------------------------------
--PROMPT 1 = &&1
--PROMPT 2 = &&2

-- SQL*PLUS SETTINGS ------------------------------------------------

SET SERVEROUTPUT ON FORMAT WRAPPED;
SET VERIFY OFF

-- SHOW ERRORS
-- SHOW USER

-- DECLARATIONS -----------------------------------------------------
DECLARE
  p_null CHAR(1);
  
  v_arg1   VARCHAR2(30) := '';
  v_arg2   VARCHAR2(30) := '';

  v_schema VARCHAR2(30) := '';
  v_table  VARCHAR2(30) := '';
  
  sql_stmt VARCHAR2(1024) := '';
  
  n_tab_seg_y   INTEGER; -- #tables with segment created
  n_tab_seg_y_b INTEGER; -- #tables with segment created before
  n_tab_seg_y_a INTEGER; -- #tables with segment created after 

  n_tab_seg_n   INTEGER; -- #tables with segment deferred
  n_tab_seg_n_b INTEGER; -- #tables with segment deferred before
  n_tab_seg_n_a INTEGER; -- #tables with segment deferred after 
  
  v_seg_created dba_tables.segment_created%type;

  PROCEDURE MSG_OUT (msg_txt IN VARCHAR2 DEFAULT ' ')
  IS
  BEGIN
    DBMS_OUTPUT.PUT_LINE(msg_txt);
  END MSG_OUT;

  PROCEDURE SHOW_USAGE
  IS
  BEGIN
    MSG_OUT('Usage');
    MSG_OUT('-----');
    MSG_OUT('In order to materialize deferred segments for a given schema, run ');
    MSG_OUT('  SQL> @create_deferred_segments <schema name>');
    MSG_OUT('To order to materialize a deferred segment for a given table, run ');
    MSG_OUT('  SQL> @create_deferred_segments <schema name> <table name>');
  END SHOW_USAGE;

  PROCEDURE SHOW_INFO_1 (msg_txt IN VARCHAR2 DEFAULT ' ')
  IS
  BEGIN
  	MSG_OUT(' ');
  	MSG_OUT(msg_txt);
    MSG_OUT('----------------------------------------------');
    MSG_OUT('Schema name                  : ' || v_schema   );
    MSG_OUT('#tables with segment created : ' || n_tab_seg_y);
    MSG_OUT('#tables with segment deferred: ' || n_tab_seg_n);
  END SHOW_INFO_1;

  PROCEDURE SHOW_INFO_2 (msg_txt IN VARCHAR2 DEFAULT ' ')
  IS
  BEGIN
  	MSG_OUT(' ');
  	MSG_OUT(msg_txt);
    MSG_OUT('----------------------------------------------');
    MSG_OUT('Schema name                  : ' || v_schema   );
    MSG_OUT('Table name                   : ' || v_table    );
    MSG_OUT('Segment created?             : ' || v_seg_created);
  END SHOW_INFO_2;

  PROCEDURE CHECK_SCHEMA
  IS
  BEGIN
  	SELECT NULL INTO p_null from dba_users where username = v_schema;
  	  EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE_APPLICATION_ERROR (-20000, 'Schema ''' || v_schema || ''' does not exist.');
        WHEN OTHERS        THEN NULL;
  END CHECK_SCHEMA;

  PROCEDURE CHECK_TABLE
  IS
  BEGIN
  	SELECT NULL INTO p_null from dba_tables where owner = v_schema and table_name = v_table;
  	  EXCEPTION
        WHEN NO_DATA_FOUND THEN RAISE_APPLICATION_ERROR (-20000, 'Table ''' || v_schema || '.' || v_table || ''' does not exist.');
        WHEN OTHERS        THEN NULL;
  END CHECK_TABLE;

---------------------------------------------------------------------
-- MAIN PROGRAM -----------------------------------------------------
---------------------------------------------------------------------
BEGIN
  DBMS_OUTPUT.ENABLE(900000);
  
  v_arg1 := lower('&&1'); v_schema := upper(v_arg1);
  v_arg2 := lower('&&2'); v_table  := upper(v_arg2);

  IF (v_schema IS NULL) THEN 
    -- no schema name specified
    MSG_OUT('No argument given.');
    SHOW_USAGE;
    GOTO OPERATION_END;
  END IF;

  IF (v_schema IS NOT NULL) THEN
    IF (v_table IS NULL) THEN
      -- ------------------------
      -- schema name specified --
      -- ------------------------
      CHECK_SCHEMA;

      -- operation
    	MSG_OUT(' ');
    	MSG_OUT('Operation: Materializing deferred segments in schema ' || v_schema);
      
      SELECT count(*) INTO n_tab_seg_y from dba_tables where owner = v_schema and segment_created = 'YES';
      SELECT count(*) INTO n_tab_seg_n from dba_tables where owner = v_schema and segment_created = 'NO';
      n_tab_seg_y_b := n_tab_seg_y; -- store number
      n_tab_seg_n_b := n_tab_seg_n; -- store number
      SHOW_INFO_1('Before materializing deferred segments');
      
      sql_stmt := 'DBMS_SPACE_ADMIN.MATERIALIZE_DEFERRED_SEGMENTS' || '(''' || v_schema || ''')';
    	MSG_OUT(' ');
      MSG_OUT('Executing ' || sql_stmt);
      DBMS_SPACE_ADMIN.MATERIALIZE_DEFERRED_SEGMENTS(v_schema);
      
      SELECT count(*) INTO n_tab_seg_y from dba_tables where owner = v_schema and segment_created = 'YES';
      SELECT count(*) INTO n_tab_seg_n from dba_tables where owner = v_schema and segment_created = 'NO';
      n_tab_seg_y_a := n_tab_seg_y; -- store number
      n_tab_seg_n_a := n_tab_seg_n; -- store number
      SHOW_INFO_1('After materializing deferred segments');
      
      MSG_OUT(' ');
      MSG_OUT('Result');
      MSG_OUT('----------------------------------------------');
      MSG_OUT('#tables with segment created  [before/now]: ' || n_tab_seg_y_b || ' / ' || n_tab_seg_y_a);
      MSG_OUT('#tables with segment deferred [before/now]: ' || n_tab_seg_n_b || ' / ' || n_tab_seg_n_a);
    ELSE
      -- ---------------------------------------
      -- schema name and table name specified --
      -- ---------------------------------------
      CHECK_SCHEMA;
      CHECK_TABLE;

      -- operation
    	MSG_OUT(' ');
    	MSG_OUT('Operation: Materializing deferred segment of ' || v_schema || '.' || v_table);

      SELECT segment_created INTO v_seg_created from dba_tables where owner = v_schema and table_name = v_table;
      SHOW_INFO_2;

      sql_stmt := 'DBMS_SPACE_ADMIN.MATERIALIZE_DEFERRED_SEGMENTS' || '(' || '''' || v_schema || '''' || ', ' || '''' || v_table || '''' || ')';
    	MSG_OUT(' ');
      MSG_OUT('Executing ' || sql_stmt);
      DBMS_SPACE_ADMIN.MATERIALIZE_DEFERRED_SEGMENTS(v_schema, v_table);

      SELECT segment_created INTO v_seg_created from dba_tables where owner = v_schema and table_name = v_table;
      SHOW_INFO_2;
    END IF;
  END IF;

  <<OPERATION_END>>
  MSG_OUT;
  MSG_OUT('Operation finished successfully.');
END;
/

-- INPUT VARIABLES - RESET ------------------------------------------
SET VERIFY       OFF
SET HEADING      OFF
SET FEEDBACK     OFF
SET SERVEROUTPUT OFF

COL 1 NEW_VALUE 1
COL 2 NEW_VALUE 2

SELECT NULL "1", NULL "2" FROM DUAL WHERE ROWNUM = 1
/
  
-- RESTORE SQL*PLUS SETTINGS ----------------------------------------
SET LINESIZE     80
SET PAGESIZE     14
SET VERIFY       ON
SET HEADING      ON
SET FEEDBACK     ON
SET SERVEROUTPUT ON

-- END --------------------------------------------------------------
-- SHOW ERRORS
